﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows.Controls.ScheduleView;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.ScheduleView.ICalendar;
using Microsoft.LightSwitch.Presentation;

namespace LightSwitchApplication
{
    public partial class RadScheduleViewControl : UserControl
    {
        public List<Appointment> _CoreAppointments;
        public List<ICategory> categories;
        public List<ITimeMarker> timemarkers;
		public List<ScheduleViewResource> scheduleViewResources;

        public RadScheduleViewControl()
        {
            InitializeComponent();
            xScheduleView.Loaded += new RoutedEventHandler(xScheduleView_Loaded);
        }

        void xScheduleView_Loaded(object sender, RoutedEventArgs e)
         {
            _CoreAppointments = new List<Appointment>();

            categories = new List<ICategory>();

            foreach (ICategory icat in xScheduleView.CategoriesSource)
            {
                categories.Add(icat);
            }

            timemarkers = new List<ITimeMarker>();

            foreach (ITimeMarker marker in xScheduleView.TimeMarkersSource)
            {
                timemarkers.Add(marker);
            }

			var objDataContext = (IContentItem)this.DataContext;

			var Screen = (Microsoft.LightSwitch.Client.IScreenObject)objDataContext.Screen;

			Screen.Details.Dispatcher.BeginInvoke(() =>
			{
				var resourceType_Service = new ResourceType("Service");
				var resourceType_Local = new ResourceType("Local");

				var serviceResources = (Screen as EditableScheduleViewApptsGrid).ServiceResources.ToList();

				this.scheduleViewResources = serviceResources;

				foreach (var resource in serviceResources)
				{
					var res = new Resource(resource.ResourceName, resource.ResourceType);

					if (res.ResourceType == "Local")
					{
						resourceType_Local.Resources.Add(res);
					}
					else if (res.ResourceType == "Service")
					{
						resourceType_Service.Resources.Add(res);
					}
				}

				var resourceTypes = new List<ResourceType>();
				resourceTypes.Add(resourceType_Service);
				resourceTypes.Add(resourceType_Local);


				Dispatcher.BeginInvoke(() =>
				{
					xScheduleView.ResourceTypesSource = resourceTypes;
				});

				var GroupDescriptions = new GroupDescriptionCollection();

				ResourceGroupDescription serviceGrp = new ResourceGroupDescription();
				serviceGrp.ResourceType = "Service";
				serviceGrp.ShowNullGroup = false;
				GroupDescriptions.Add(serviceGrp);

				ResourceGroupDescription locationGrp = new ResourceGroupDescription();
				locationGrp.ResourceType = "Local";
				locationGrp.ShowNullGroup = false;
				GroupDescriptions.Add(locationGrp);

				Dispatcher.BeginInvoke(() =>
				{
					xScheduleView.GroupDescriptionsSource = GroupDescriptions;
				});

			});
        }

        public void GetLSAppointments(List<ScheduleViewAppt> svAppts)
        {
			this._CoreAppointments = new List<Appointment>();

            foreach (ScheduleViewAppt sva in svAppts)
            {
                Appointment appt = MakeAppointment(sva);

                _CoreAppointments.Add(appt);
            }

            Dispatcher.BeginInvoke(() =>
            {
                xScheduleView.AppointmentsSource = _CoreAppointments;
            });

        }

        private void xScheduleView_AppointmentCreated(object sender, Telerik.Windows.Controls.AppointmentCreatedEventArgs e)
        {
            var objDataContext = (IContentItem)this.DataContext;

            var Screen = (Microsoft.LightSwitch.Client.IScreenObject)objDataContext.Screen;

            Screen.Details.Dispatcher.BeginInvoke(() =>
                {
                    ScheduleViewAppt sva = (Screen as EditableScheduleViewApptsGrid).ScheduleViewAppts.AddNew();

                    Appointment appt = e.CreatedAppointment as Appointment;

                    UpdateAppointment(sva, appt);
                });
        }

        private void xScheduleView_AppointmentDeleted(object sender, Telerik.Windows.Controls.AppointmentDeletedEventArgs e)
        {                  
            var objDataContext = (IContentItem)this.DataContext;

            var Screen = (Microsoft.LightSwitch.Client.IScreenObject)objDataContext.Screen;

            Screen.Details.Dispatcher.BeginInvoke(() =>
            {
                Appointment appt = e.Appointment as Appointment;

                ScheduleViewAppt sva = (Screen as EditableScheduleViewApptsGrid).ScheduleViewAppts.Where(x => x.ApptGuid == appt.UniqueId).SingleOrDefault();

                if (sva != null)
                {
                    sva.Delete();
                }
            });    
        }

        private void xScheduleView_AppointmentEdited(object sender, Telerik.Windows.Controls.AppointmentEditedEventArgs e)
        {
            var objDataContext = (IContentItem)this.DataContext;

            var Screen = (Microsoft.LightSwitch.Client.IScreenObject)objDataContext.Screen;

            Screen.Details.Dispatcher.BeginInvoke(() =>
            {
                Appointment appt = e.Appointment as Appointment;

                ScheduleViewAppt sva = (Screen as EditableScheduleViewApptsGrid).ScheduleViewAppts.Where(x => x.ApptGuid == appt.UniqueId).SingleOrDefault();

                if (sva != null)
                {
                    UpdateAppointment(sva, appt);
                }
            });
        }

        private Appointment MakeAppointment(ScheduleViewAppt sva)
        {
            Appointment appt = new Appointment();

            appt.Subject = sva.Subject;
            appt.UniqueId = sva.ApptGuid;
            appt.Start = sva.StartDate;
            appt.End = sva.EndDate;
            appt.Body = sva.Description;
            appt.IsAllDayEvent = Convert.ToBoolean(sva.IsAllDay);
            appt.Location = sva.Location;
            appt.Url = sva.URL;

			foreach (var apptToResource in sva.ApptToResources)
			{
				var res = new Resource(apptToResource.ScheduleViewResource.ResourceName, 
					apptToResource.ScheduleViewResource.ResourceType);
				appt.Resources.Add(res);
			}

            if (sva.Category != null)
            {
                appt.Category = categories.Where(x => x.CategoryName == sva.Category).SingleOrDefault();
            }

            if (sva.Importance != null)
            {
                appt.Importance = (Importance)Enum.Parse(typeof(Importance), sva.Importance, true);
            }

            if (sva.TimeMarker != null)
            {
                appt.TimeMarker = timemarkers.Where(x => x.TimeMarkerName == sva.TimeMarker).SingleOrDefault();
            }

            if (sva.RecurrenceRule != null)
            {
                RecurrencePattern pattern = new RecurrencePattern();
                RecurrencePatternHelper.TryParseRecurrencePattern(sva.RecurrenceRule, out pattern);
                appt.RecurrenceRule = new RecurrenceRule(pattern);
            }

            return appt;
        }

        private void UpdateAppointment(ScheduleViewAppt sva, Appointment appt)
        {
            sva.Subject = appt.Subject;
            sva.ApptGuid = appt.UniqueId;
            sva.StartDate = appt.Start;
            sva.EndDate = appt.End;
            sva.Description = appt.Body;
            sva.IsAllDay = appt.IsAllDayEvent;
            sva.Location = appt.Location;
            sva.URL = appt.Url;

			var itemsToRemove = sva.ApptToResources;

			var c = itemsToRemove.Count();
			for(int i=0; i <c; i++)
			{
				//sva.ApptToResources.Remove(itemsToRemove.ElementAt(i));
				itemsToRemove.First().Delete();
			}

			foreach (var res in appt.Resources)
			{
				sva.ApptToResources.AddNew();
				var resourceToAppt = sva.ApptToResources.Last();
				resourceToAppt.ScheduleViewResource = this.scheduleViewResources.First(x =>
					x.ResourceName == res.ResourceName && x.ResourceType == res.ResourceType);
			}

            if (appt.Category != null)
            {
                sva.Category = appt.Category.CategoryName;
            }

            if (appt.Importance != null)
            {
                sva.Importance = appt.Importance.ToString();
            }

            if (appt.TimeMarker != null)
            {
                sva.TimeMarker = appt.TimeMarker.TimeMarkerName;
            }

            if (appt.RecurrenceRule != null)
            {
                sva.RecurrenceRule = RecurrencePatternHelper.RecurrencePatternToString(appt.RecurrenceRule.Pattern);
            }
        }
    }
}
